@@ -114,49 +114,15 @@ static inline u8 FAN_TO_REG(long rpm, int div)
 #define VID_FROM_REG(val) ((val)==0x1f?0:(val)>=0x10?510-(val)*10:\
                            205-(val)*5)
 
-/* Initial values */
-#define GL520_INIT_TEMP_OVER 600
-#define GL520_INIT_TEMP_HYST 500
-#define GL520_INIT_FAN_MIN_1 3000
-#define GL520_INIT_FAN_MIN_2 3000
-
-/* These are somewhat sane */
-#define GL520_INIT_VIN_1 330	/* 3.3 V */
-#define GL520_INIT_VIN_2 286	/* 12 V */
-#define GL520_INIT_VIN_3 260	/* Vcore */
-#define GL520_INIT_VIN_4 160	/* -12 V */
-#define GL520_INIT_VDD 500	/* 5 V */
-
-#define GL520_INIT_PERCENTAGE 10
-
-#define GL520_INIT_VIN_MIN_1 \
-        (GL520_INIT_VIN_1 - GL520_INIT_VIN_1 * GL520_INIT_PERCENTAGE / 100)
-#define GL520_INIT_VIN_MAX_1 \
-        (GL520_INIT_VIN_1 + GL520_INIT_VIN_1 * GL520_INIT_PERCENTAGE / 100)
-#define GL520_INIT_VIN_MIN_2 \
-        (GL520_INIT_VIN_2 - GL520_INIT_VIN_2 * GL520_INIT_PERCENTAGE / 100)
-#define GL520_INIT_VIN_MAX_2 \
-        (GL520_INIT_VIN_2 + GL520_INIT_VIN_2 * GL520_INIT_PERCENTAGE / 100)
-#define GL520_INIT_VIN_MIN_3 \
-        (GL520_INIT_VIN_3 - GL520_INIT_VIN_3 * GL520_INIT_PERCENTAGE / 100)
-#define GL520_INIT_VIN_MAX_3 \
-        (GL520_INIT_VIN_3 + GL520_INIT_VIN_3 * GL520_INIT_PERCENTAGE / 100)
-#define GL520_INIT_VDD_MIN \
-        (GL520_INIT_VDD - GL520_INIT_VDD * GL520_INIT_PERCENTAGE / 100)
-#define GL520_INIT_VDD_MAX \
-        (GL520_INIT_VDD + GL520_INIT_VDD * GL520_INIT_PERCENTAGE / 100)
-
-
 /* Each client has this additional data */
 struct gl520_data {
+	struct i2c_client client;
 	int sysctl_id;
 	enum chips type;
 
 	struct semaphore update_lock;
 	char valid;		/* !=0 if following fields are valid */
 	unsigned long last_updated;	/* In jiffies */
-	unsigned long last_updated_v00;
-	/* In jiffies (used only by rev00 chips) */
 
 	u8 voltage[5];		/* Register values; [0] = VDD */
 	u8 voltage_min[5];	/* Register values; [0] = VDD */
@@ -179,7 +145,6 @@ static int gl520_detect(struct i2c_adapter *adapter, int address,
 static void gl520_init_client(struct i2c_client *client);
 static int gl520_detach_client(struct i2c_client *client);
 
-static u16 swap_bytes(u16 val);
 static int gl520_read_value(struct i2c_client *client, u8 reg);
 static int gl520_write_value(struct i2c_client *client, u8 reg, u16 value);
 static void gl520_update_client(struct i2c_client *client);
@@ -281,9 +246,6 @@ static ctl_table gl520_dir_table_template[] = {
 	{0}
 };
 
-/* I choose here for semi-static GL520SM allocation. Complete dynamic
-   allocation could also be used; the code needed for this would probably
-   take more memory than the datastructure takes now. */
 static int gl520_id = 0;
 
 static int gl520_attach_adapter(struct i2c_adapter *adapter)
@@ -319,14 +281,12 @@ static int gl520_detect(struct i2c_adapter *adapter, int address,
 	   client structure, even though we cannot fill it completely yet.
 	   But it allows us to access gl520_{read,write}_value. */
 
-	if (!(new_client = kmalloc(sizeof(struct i2c_client) +
-				   sizeof(struct gl520_data),
-				   GFP_KERNEL))) {
+	if (!(data = kmalloc(sizeof(struct gl520_data), GFP_KERNEL))) {
 		err = -ENOMEM;
 		goto ERROR0;
 	}
 
-	data = (struct gl520_data *) (new_client + 1);
+	new_client = &data->client;
 	new_client->addr = address;
 	new_client->data = data;
 	new_client->adapter = adapter;
@@ -379,9 +339,15 @@ static int gl520_detect(struct i2c_adapter *adapter, int address,
 	data->sysctl_id = i;
 
 	/* Initialize the GL520SM chip */
-	data->two_temps = 1;
 	data->alarm_mask = 0xff;
 	gl520_init_client(new_client);
+	if (data->two_temps)
+		data->voltage_max[4] = data->voltage_min[4] =
+			data->voltage[4] = 0;
+	else
+		data->temp_hyst[1] = data->temp_over[1] =
+			data->temp[1] = 0;
+
 	return 0;
 
 /* OK, this is not exactly good programming practice, usually. But it is
@@ -391,58 +357,30 @@ static int gl520_detect(struct i2c_adapter *adapter, int address,
 	i2c_detach_client(new_client);
       ERROR3:
       ERROR1:
-	kfree(new_client);
+	kfree(data);
       ERROR0:
 	return err;
 }
 
 
-/* Called when we have found a new GL520SM. It should set limits, etc. */
+/* Called when we have found a new GL520SM. */
 static void gl520_init_client(struct i2c_client *client)
 {
-	/* Power-on defaults (bit 7=1) */
-	gl520_write_value(client, GL520_REG_CONF, 0x80);
+	struct gl520_data *data = (struct gl520_data *)(client->data);
+	u8 oldconf, conf;
 
-	/* No noisy output (bit 2=1), Comparator mode (bit 3=0), two fans (bit4=0),
-	   standby mode (bit6=0) */
-	gl520_write_value(client, GL520_REG_CONF, 0x04);
+	conf = oldconf = gl520_read_value(client, GL520_REG_CONF);
+	data->two_temps = !(conf & 0x10);
 
-	/* Never interrupts */
-	gl520_write_value(client, GL520_REG_MASK, 0x00);
+	/* If IRQ# is disabled, we can safely force comparator mode */
+	if (!(conf & 0x20))
+		conf &= 0xf7;
 
-	gl520_write_value(client, GL520_REG_TEMP1_HYST,
-			  TEMP_TO_REG(GL520_INIT_TEMP_HYST));
-	gl520_write_value(client, GL520_REG_TEMP1_OVER,
-			  TEMP_TO_REG(GL520_INIT_TEMP_OVER));
+	/* Enable monitoring if needed */
+	conf |= 0x40;
 
-	/* We set Temp2, but not Vin4. */
-	gl520_write_value(client, GL520_REG_TEMP2_HYST,
-			  TEMP_TO_REG(GL520_INIT_TEMP_HYST));
-	gl520_write_value(client, GL520_REG_TEMP2_OVER,
-			  TEMP_TO_REG(GL520_INIT_TEMP_OVER));
-
-	gl520_write_value(client, GL520_REG_MISC, (DIV_TO_REG(2) << 6) |
-			  (DIV_TO_REG(2) << 4));
-	gl520_write_value(client, GL520_REG_FAN_LIMIT,
-			  (FAN_TO_REG(GL520_INIT_FAN_MIN_1, 2) << 8) |
-			  FAN_TO_REG(GL520_INIT_FAN_MIN_2, 2));
-
-	gl520_write_value(client, GL520_REG_VIN1_LIMIT,
-			  (IN_TO_REG(GL520_INIT_VIN_MAX_1) << 8) |
-			  IN_TO_REG(GL520_INIT_VIN_MIN_1));
-	gl520_write_value(client, GL520_REG_VIN2_LIMIT,
-			  (IN_TO_REG(GL520_INIT_VIN_MAX_2) << 8) |
-			  IN_TO_REG(GL520_INIT_VIN_MIN_2));
-	gl520_write_value(client, GL520_REG_VIN3_LIMIT,
-			  (IN_TO_REG(GL520_INIT_VIN_MAX_3) << 8) |
-			  IN_TO_REG(GL520_INIT_VIN_MIN_3));
-	gl520_write_value(client, GL520_REG_VDD_LIMIT,
-			  (VDD_TO_REG(GL520_INIT_VDD_MAX) << 8) |
-			  VDD_TO_REG(GL520_INIT_VDD_MIN));
-
-	/* Clear status register (bit 5=1), start (bit6=1) */
-	gl520_write_value(client, GL520_REG_CONF, 0x24);
-	gl520_write_value(client, GL520_REG_CONF, 0x44);
+	if (conf != oldconf)
+		gl520_write_value(client, GL520_REG_CONF, conf);
 }
 
 static int gl520_detach_client(struct i2c_client *client)
@@ -458,24 +396,19 @@ static int gl520_detach_client(struct i2c_client *client)
 		return err;
 	}
 
-	kfree(client);
+	kfree(client->data);
 
 	return 0;
 }
 
 
-static u16 swap_bytes(u16 val)
-{
-	return (val >> 8) | (val << 8);
-}
-
 /* Registers 0x07 to 0x0c are word-sized, others are byte-sized 
    GL520 uses a high-byte first convention, which is exactly opposite to
    the usual practice. */
 static int gl520_read_value(struct i2c_client *client, u8 reg)
 {
 	if ((reg >= 0x07) && (reg <= 0x0c))
-		return swap_bytes(i2c_smbus_read_word_data(client, reg));
+		return swab16(i2c_smbus_read_word_data(client, reg));
 	else
 		return i2c_smbus_read_byte_data(client, reg);
 }
@@ -486,8 +419,7 @@ static int gl520_read_value(struct i2c_client *client, u8 reg)
 static int gl520_write_value(struct i2c_client *client, u8 reg, u16 value)
 {
 	if ((reg >= 0x07) && (reg <= 0x0c))
-		return i2c_smbus_write_word_data(client, reg,
-						 swap_bytes(value));
+		return i2c_smbus_write_word_data(client, reg, swab16(value));
 	else
 		return i2c_smbus_write_byte_data(client, reg, value);
 }
@@ -555,18 +487,21 @@ static void gl520_update_client(struct i2c_client *client)
 		    gl520_read_value(client, GL520_REG_VIN3);
 
 		/* Temp1 and Vin4 are the same input */
-		data->temp[1] = gl520_read_value(client, GL520_REG_TEMP2);
-		data->temp_over[1] =
-		    gl520_read_value(client, GL520_REG_TEMP2_OVER);
-		data->temp_hyst[1] =
-		    gl520_read_value(client, GL520_REG_TEMP2_HYST);
-
-		data->voltage[4] =
-		    gl520_read_value(client, GL520_REG_VIN4);
-		data->voltage_min[4] =
-		    gl520_read_value(client, GL520_REG_VIN4_MIN);
-		data->voltage_max[4] =
-		    gl520_read_value(client, GL520_REG_VIN4_MAX);
+		if (data->two_temps) {
+			data->temp[1] =
+			    gl520_read_value(client, GL520_REG_TEMP2);
+			data->temp_over[1] =
+			    gl520_read_value(client, GL520_REG_TEMP2_OVER);
+			data->temp_hyst[1] =
+			    gl520_read_value(client, GL520_REG_TEMP2_HYST);
+		} else {
+			data->voltage[4] =
+			    gl520_read_value(client, GL520_REG_VIN4);
+			data->voltage_min[4] =
+			    gl520_read_value(client, GL520_REG_VIN4_MIN);
+			data->voltage_max[4] =
+			    gl520_read_value(client, GL520_REG_VIN4_MAX);
+		}
 
 		data->last_updated = jiffies;
 		data->valid = 1;
@@ -591,7 +526,7 @@ void gl520_temp(struct i2c_client *client, int operation, int ctl_name,
 		results[2] = TEMP_FROM_REG(data->temp[nr]);
 		*nrels_mag = 3;
 	} else if (operation == SENSORS_PROC_REAL_WRITE) {
-		if ((nr == 0) && (!data->two_temps))
+		if ((nr == 1) && (!data->two_temps))
 			return;
 		regnr =
 		    nr == 0 ? GL520_REG_TEMP1_OVER : GL520_REG_TEMP2_OVER;
@@ -840,8 +775,12 @@ void gl520_config(struct i2c_client *client, int operation, int ctl_name,
 			if (!results[1]) {
 				old |= 0x10;
 				data->two_temps = 0;
+				data->temp_hyst[1] = data->temp_over[1] =
+					data->temp[1] = 0;
 			} else {
 				data->two_temps = 1;
+				data->voltage_max[4] = data->voltage_min[4] =
+					data->voltage[4] = 0;
 			}
 			gl520_write_value(client, GL520_REG_CONF, old);
 		}
