@@ -21,15 +21,9 @@
 
 #include <linux/module.h>
 #include <linux/slab.h>
-#include <linux/proc_fs.h>
-#include <linux/ioport.h>
-#include <linux/sysctl.h>
-#include <linux/types.h>
 #include <linux/i2c.h>
 #include <linux/i2c-proc.h>
 #include <linux/init.h>
-#include <asm/errno.h>
-#include <asm/io.h>
 #include "version.h"
 
 MODULE_LICENSE("GPL");
@@ -75,8 +69,8 @@ SENSORS_INSMOD_1(lm80);
    these macros are called: arguments may be evaluated more than once.
    Fixing this is just not worth it. */
 
-#define IN_TO_REG(val,nr) (SENSORS_LIMIT((val),0,255))
-#define IN_FROM_REG(val,nr) (val)
+#define IN_TO_REG(val) (SENSORS_LIMIT((val),0,255))
+#define IN_FROM_REG(val) (val)
 
 static inline unsigned char FAN_TO_REG(unsigned rpm, unsigned div)
 {
@@ -94,12 +88,12 @@ static inline long TEMP_FROM_REG(u16 temp)
 {
 	long res;
 
-	temp = temp >> 4;
-	if (temp < 0x0800) {
-		res = (625 * (long) temp);
-	} else {
+	temp >>= 4;
+	if (temp < 0x0800)
+		res = 625 * (long) temp;
+	else
 		res = ((long) temp - 0x01000) * 625;
-	}
+
 	return res / 100;
 }
 
@@ -114,60 +108,8 @@ static inline long TEMP_FROM_REG(u16 temp)
 #define DIV_FROM_REG(val) (1 << (val))
 #define DIV_TO_REG(val) ((val)==8?3:(val)==4?2:(val)==1?0:1)
 
-/* Initial limits */
-#define LM80_INIT_IN_0 190
-#define LM80_INIT_IN_1 190
-#define LM80_INIT_IN_2 190
-#define LM80_INIT_IN_3 190
-#define LM80_INIT_IN_4 190
-#define LM80_INIT_IN_5 190
-#define LM80_INIT_IN_6 190
-
-#define LM80_INIT_IN_PERCENTAGE 10
-
-#define LM80_INIT_IN_MIN_0 \
-        (LM80_INIT_IN_0 - LM80_INIT_IN_0 * LM80_INIT_IN_PERCENTAGE / 100)
-#define LM80_INIT_IN_MAX_0 \
-        (LM80_INIT_IN_0 + LM80_INIT_IN_0 * LM80_INIT_IN_PERCENTAGE / 100)
-#define LM80_INIT_IN_MIN_1 \
-        (LM80_INIT_IN_1 - LM80_INIT_IN_1 * LM80_INIT_IN_PERCENTAGE / 100)
-#define LM80_INIT_IN_MAX_1 \
-        (LM80_INIT_IN_1 + LM80_INIT_IN_1 * LM80_INIT_IN_PERCENTAGE / 100)
-#define LM80_INIT_IN_MIN_2 \
-        (LM80_INIT_IN_2 - LM80_INIT_IN_2 * LM80_INIT_IN_PERCENTAGE / 100)
-#define LM80_INIT_IN_MAX_2 \
-        (LM80_INIT_IN_2 + LM80_INIT_IN_2 * LM80_INIT_IN_PERCENTAGE / 100)
-#define LM80_INIT_IN_MIN_3 \
-        (LM80_INIT_IN_3 - LM80_INIT_IN_3 * LM80_INIT_IN_PERCENTAGE / 100)
-#define LM80_INIT_IN_MAX_3 \
-        (LM80_INIT_IN_3 + LM80_INIT_IN_3 * LM80_INIT_IN_PERCENTAGE / 100)
-#define LM80_INIT_IN_MIN_4 \
-        (LM80_INIT_IN_4 - LM80_INIT_IN_4 * LM80_INIT_IN_PERCENTAGE / 100)
-#define LM80_INIT_IN_MAX_4 \
-        (LM80_INIT_IN_4 + LM80_INIT_IN_4 * LM80_INIT_IN_PERCENTAGE / 100)
-#define LM80_INIT_IN_MIN_5 \
-        (LM80_INIT_IN_5 - LM80_INIT_IN_5 * LM80_INIT_IN_PERCENTAGE / 100)
-#define LM80_INIT_IN_MAX_5 \
-        (LM80_INIT_IN_5 + LM80_INIT_IN_5 * LM80_INIT_IN_PERCENTAGE / 100)
-#define LM80_INIT_IN_MIN_6 \
-        (LM80_INIT_IN_6 - LM80_INIT_IN_6 * LM80_INIT_IN_PERCENTAGE / 100)
-#define LM80_INIT_IN_MAX_6 \
-        (LM80_INIT_IN_6 + LM80_INIT_IN_6 * LM80_INIT_IN_PERCENTAGE / 100)
-
-#define LM80_INIT_FAN_MIN_1 3000
-#define LM80_INIT_FAN_MIN_2 3000
-
-#define LM80_INIT_TEMP_OS_MAX 600
-#define LM80_INIT_TEMP_OS_HYST 500
-#define LM80_INIT_TEMP_HOT_MAX 700
-#define LM80_INIT_TEMP_HOT_HYST 600
-
-
-/* For each registered LM80, we need to keep some data in memory. That
-   data is pointed to by lm80_list[NR]->data. The structure itself is
-   dynamically allocated, at the same time when a new lm80 client is
-   allocated. */
 struct lm80_data {
+	struct i2c_client client;
 	int sysctl_id;
 
 	struct semaphore update_lock;
@@ -195,9 +137,8 @@ static int lm80_detect(struct i2c_adapter *adapter, int address,
 		       unsigned short flags, int kind);
 static int lm80_detach_client(struct i2c_client *client);
 
-static int lm80_read_value(struct i2c_client *client, u8 register);
-static int lm80_write_value(struct i2c_client *client, u8 register,
-			    u8 value);
+static int lm80_read_value(struct i2c_client *client, u8 reg);
+static int lm80_write_value(struct i2c_client *client, u8 reg, u8 value);
 static void lm80_update_client(struct i2c_client *client);
 static void lm80_init_client(struct i2c_client *client);
 
@@ -321,14 +262,12 @@ int lm80_detect(struct i2c_adapter *adapter, int address,
 	/* OK. For now, we presume we have a valid client. We now create the
 	   client structure, even though we cannot fill it completely yet.
 	   But it allows us to access lm80_{read,write}_value. */
-	if (!(new_client = kmalloc(sizeof(struct i2c_client) +
-				   sizeof(struct lm80_data),
-				   GFP_KERNEL))) {
+	if (!(data = kmalloc(sizeof(struct lm80_data), GFP_KERNEL))) {
 		err = -ENOMEM;
 		goto ERROR0;
 	}
 
-	data = (struct lm80_data *) (new_client + 1);
+	new_client = &data->client;
 	new_client->addr = address;
 	new_client->data = data;
 	new_client->adapter = adapter;
@@ -390,7 +329,7 @@ int lm80_detect(struct i2c_adapter *adapter, int address,
 	i2c_detach_client(new_client);
       ERROR3:
       ERROR1:
-	kfree(new_client);
+	kfree(data);
       ERROR0:
 	return err;
 }
@@ -408,7 +347,7 @@ static int lm80_detach_client(struct i2c_client *client)
 		return err;
 	}
 
-	kfree(client);
+	kfree(client->data);
 
 	return 0;
 }
@@ -423,7 +362,7 @@ static int lm80_write_value(struct i2c_client *client, u8 reg, u8 value)
 	return i2c_smbus_write_byte_data(client, reg, value);
 }
 
-/* Called when we have found a new LM80. It should set limits, etc. */
+/* Called when we have found a new LM80. */
 static void lm80_init_client(struct i2c_client *client)
 {
 	/* Reset all except Watchdog values and last conversion values
@@ -433,47 +372,6 @@ static void lm80_init_client(struct i2c_client *client)
 	/* Set 11-bit temperature resolution */
 	lm80_write_value(client, LM80_REG_RES, 0x08);
 
-	lm80_write_value(client, LM80_REG_IN_MIN(0),
-			 IN_TO_REG(LM80_INIT_IN_MIN_0, 0));
-	lm80_write_value(client, LM80_REG_IN_MAX(0),
-			 IN_TO_REG(LM80_INIT_IN_MAX_0, 0));
-	lm80_write_value(client, LM80_REG_IN_MIN(1),
-			 IN_TO_REG(LM80_INIT_IN_MIN_1, 1));
-	lm80_write_value(client, LM80_REG_IN_MAX(1),
-			 IN_TO_REG(LM80_INIT_IN_MAX_1, 1));
-	lm80_write_value(client, LM80_REG_IN_MIN(2),
-			 IN_TO_REG(LM80_INIT_IN_MIN_2, 2));
-	lm80_write_value(client, LM80_REG_IN_MAX(2),
-			 IN_TO_REG(LM80_INIT_IN_MAX_2, 2));
-	lm80_write_value(client, LM80_REG_IN_MIN(3),
-			 IN_TO_REG(LM80_INIT_IN_MIN_3, 3));
-	lm80_write_value(client, LM80_REG_IN_MAX(3),
-			 IN_TO_REG(LM80_INIT_IN_MAX_3, 3));
-	lm80_write_value(client, LM80_REG_IN_MIN(4),
-			 IN_TO_REG(LM80_INIT_IN_MIN_4, 4));
-	lm80_write_value(client, LM80_REG_IN_MAX(4),
-			 IN_TO_REG(LM80_INIT_IN_MAX_4, 4));
-	lm80_write_value(client, LM80_REG_IN_MIN(5),
-			 IN_TO_REG(LM80_INIT_IN_MIN_5, 5));
-	lm80_write_value(client, LM80_REG_IN_MAX(5),
-			 IN_TO_REG(LM80_INIT_IN_MAX_5, 5));
-	lm80_write_value(client, LM80_REG_IN_MIN(6),
-			 IN_TO_REG(LM80_INIT_IN_MIN_6, 6));
-	lm80_write_value(client, LM80_REG_IN_MAX(6),
-			 IN_TO_REG(LM80_INIT_IN_MAX_6, 6));
-	lm80_write_value(client, LM80_REG_FAN1_MIN,
-			 FAN_TO_REG(LM80_INIT_FAN_MIN_1, 2));
-	lm80_write_value(client, LM80_REG_FAN2_MIN,
-			 FAN_TO_REG(LM80_INIT_FAN_MIN_2, 2));
-	lm80_write_value(client, LM80_REG_TEMP_HOT_MAX,
-			 TEMP_LIMIT_TO_REG(LM80_INIT_TEMP_OS_MAX));
-	lm80_write_value(client, LM80_REG_TEMP_HOT_HYST,
-			 TEMP_LIMIT_TO_REG(LM80_INIT_TEMP_OS_HYST));
-	lm80_write_value(client, LM80_REG_TEMP_OS_MAX,
-			 TEMP_LIMIT_TO_REG(LM80_INIT_TEMP_OS_MAX));
-	lm80_write_value(client, LM80_REG_TEMP_OS_HYST,
-			 TEMP_LIMIT_TO_REG(LM80_INIT_TEMP_OS_HYST));
-
 	/* Start monitoring */
 	lm80_write_value(client, LM80_REG_CONFIG, 0x01);
 }
@@ -554,18 +452,18 @@ void lm80_in(struct i2c_client *client, int operation, int ctl_name,
 		*nrels_mag = 2;
 	else if (operation == SENSORS_PROC_REAL_READ) {
 		lm80_update_client(client);
-		results[0] = IN_FROM_REG(data->in_min[nr], nr);
-		results[1] = IN_FROM_REG(data->in_max[nr], nr);
-		results[2] = IN_FROM_REG(data->in[nr], nr);
+		results[0] = IN_FROM_REG(data->in_min[nr]);
+		results[1] = IN_FROM_REG(data->in_max[nr]);
+		results[2] = IN_FROM_REG(data->in[nr]);
 		*nrels_mag = 3;
 	} else if (operation == SENSORS_PROC_REAL_WRITE) {
 		if (*nrels_mag >= 1) {
-			data->in_min[nr] = IN_TO_REG(results[0], nr);
+			data->in_min[nr] = IN_TO_REG(results[0]);
 			lm80_write_value(client, LM80_REG_IN_MIN(nr),
 					 data->in_min[nr]);
 		}
 		if (*nrels_mag >= 2) {
-			data->in_max[nr] = IN_TO_REG(results[1], nr);
+			data->in_max[nr] = IN_TO_REG(results[1]);
 			lm80_write_value(client, LM80_REG_IN_MAX(nr),
 					 data->in_max[nr]);
 		}
