@@ -24,29 +24,27 @@
     Supports following chips:
 
     Chip	#vin	#fanin	#pwm	#temp	wchipid	vendid	i2c	ISA
-    as99127f	7	3	1?	3	0x31	0x12c3	yes	no
+    as99127f	7	3	0	3	0x31	0x12c3	yes	no
     as99127f rev.2 (type name = as99127f)	0x31	0x5ca3	yes	no
-    asb100 "bach" (type_name = as99127f)	0x31	0x0694	yes	no
-    w83627hf	9	3	2	3	0x20	0x5ca3	yes	yes(LPC)
+    w83627hf	9	3	2	3	0x21	0x5ca3	yes	yes(LPC)
     w83697hf	8	2	2	2	0x60	0x5ca3	no	yes(LPC)
     w83781d	7	3	0	3	0x10-1	0x5ca3	yes	yes
     w83782d	9	3	2-4	3	0x30	0x5ca3	yes	yes
     w83783s	5-6	3	2	1-2	0x40	0x5ca3	yes	no
-    w83791d	10	5	5	3	0x70	0x5ca3	yes	no
+    w83791d	10	5	5	3	0x71	0x5ca3	yes	no
 
 */
 
 #include <linux/module.h>
 #include <linux/slab.h>
 #include <linux/ioport.h>
-#include <linux/types.h>
 #include <linux/i2c.h>
 #include <linux/i2c-proc.h>
 #include <linux/init.h>
-#include <asm/errno.h>
 #include <asm/io.h>
 #include "version.h"
 #include "sensors_vid.h"
+#include "lm75.h"
 
 /* RT Table support #defined so we can take it out if it gets bothersome */
 #define W83781D_RT 1
@@ -202,10 +200,6 @@ static inline u8 FAN_TO_REG(long rpm, int div)
                                                  ((val)+5)/10),0,255))
 #define TEMP_FROM_REG(val) (((val)>0x80?(val)-0x100:(val))*10)
 
-#define TEMP_ADD_TO_REG(val)   (SENSORS_LIMIT(((((val) + 2) / 5) << 7),\
-                                              0,0xffff))
-#define TEMP_ADD_FROM_REG(val) (((val) >> 7) * 5)
-
 #define AS99127_TEMP_ADD_TO_REG(val) (SENSORS_LIMIT((((((val) + 2)*4)/10) \
                                                << 7),0,0xffff))
 #define AS99127_TEMP_ADD_FROM_REG(val) ((((val) >> 7) * 10) / 4)
@@ -213,8 +207,8 @@ static inline u8 FAN_TO_REG(long rpm, int div)
 #define ALARMS_FROM_REG(val) (val)
 #define PWM_FROM_REG(val) (val)
 #define PWM_TO_REG(val) (SENSORS_LIMIT((val),0,255))
-#define BEEPS_FROM_REG(val) (val)
-#define BEEPS_TO_REG(val) ((val) & 0xffffff)
+#define BEEPS_FROM_REG(val,type) ((type)==as99127f?(val)^0x7FFF:(val))
+#define BEEPS_TO_REG(val,type) ((type)==as99127f?(~(val))&0x7FFF:(val)&0xffffff)
 
 #define BEEP_ENABLE_TO_REG(val)   ((val)?1:0)
 #define BEEP_ENABLE_FROM_REG(val) ((val)?1:0)
@@ -234,111 +228,6 @@ static inline u8 DIV_TO_REG(long val, enum chips type)
 	return ((u8) i);
 }
 
-/* Initial limits */
-#define W83781D_INIT_IN_0 (vid==3500?280:vid/10)
-#define W83781D_INIT_IN_1 (vid==3500?280:vid/10)
-#define W83781D_INIT_IN_2 330
-#define W83781D_INIT_IN_3 (((500)   * 100)/168)
-#define W83781D_INIT_IN_4 (((1200)  * 10)/38)
-#define W83781D_INIT_IN_5 (((-1200) * -604)/2100)
-#define W83781D_INIT_IN_6 (((-500)  * -604)/909)
-#define W83781D_INIT_IN_7 (((500)   * 100)/168)
-#define W83781D_INIT_IN_8 300
-#define W83781D_INIT_IN_9 250 /* 2.5 Volt for the DDR interface. */
-/* Initial limits for 782d/783s negative voltages */
-/* Note level shift. Change min/max below if you change these. */
-#define W83782D_INIT_IN_5 ((((-1200) + 1491) * 100)/514)
-#define W83782D_INIT_IN_6 ((( (-500)  + 771) * 100)/314)
-
-#define W83781D_INIT_IN_PERCENTAGE 10
-
-#define W83781D_INIT_IN_MIN_0 \
-        (W83781D_INIT_IN_0 - W83781D_INIT_IN_0 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_0 \
-        (W83781D_INIT_IN_0 + W83781D_INIT_IN_0 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MIN_1 \
-        (W83781D_INIT_IN_1 - W83781D_INIT_IN_1 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_1 \
-        (W83781D_INIT_IN_1 + W83781D_INIT_IN_1 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MIN_2 \
-        (W83781D_INIT_IN_2 - W83781D_INIT_IN_2 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_2 \
-        (W83781D_INIT_IN_2 + W83781D_INIT_IN_2 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MIN_3 \
-        (W83781D_INIT_IN_3 - W83781D_INIT_IN_3 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_3 \
-        (W83781D_INIT_IN_3 + W83781D_INIT_IN_3 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MIN_4 \
-        (W83781D_INIT_IN_4 - W83781D_INIT_IN_4 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_4 \
-        (W83781D_INIT_IN_4 + W83781D_INIT_IN_4 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MIN_5 \
-        (W83781D_INIT_IN_5 - W83781D_INIT_IN_5 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_5 \
-        (W83781D_INIT_IN_5 + W83781D_INIT_IN_5 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MIN_6 \
-        (W83781D_INIT_IN_6 - W83781D_INIT_IN_6 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_6 \
-        (W83781D_INIT_IN_6 + W83781D_INIT_IN_6 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MIN_7 \
-        (W83781D_INIT_IN_7 - W83781D_INIT_IN_7 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_7 \
-        (W83781D_INIT_IN_7 + W83781D_INIT_IN_7 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MIN_8 \
-        (W83781D_INIT_IN_8 - W83781D_INIT_IN_8 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_8 \
-        (W83781D_INIT_IN_8 + W83781D_INIT_IN_8 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MIN_9 \
-        (W83781D_INIT_IN_9 - W83781D_INIT_IN_9 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-#define W83781D_INIT_IN_MAX_9 \
-        (W83781D_INIT_IN_9 + W83781D_INIT_IN_9 * W83781D_INIT_IN_PERCENTAGE \
-         / 100)
-/* Initial limits for 782d/783s negative voltages */
-/* These aren't direct multiples because of level shift */
-/* Beware going negative - check */
-#define W83782D_INIT_IN_MIN_5_TMP \
-        (((-1200 * (100 + W83781D_INIT_IN_PERCENTAGE)) + (1491 * 100))/514)
-#define W83782D_INIT_IN_MIN_5 \
-        ((W83782D_INIT_IN_MIN_5_TMP > 0) ? W83782D_INIT_IN_MIN_5_TMP : 0)
-#define W83782D_INIT_IN_MAX_5 \
-        (((-1200 * (100 - W83781D_INIT_IN_PERCENTAGE)) + (1491 * 100))/514)
-#define W83782D_INIT_IN_MIN_6_TMP \
-        ((( -500 * (100 + W83781D_INIT_IN_PERCENTAGE)) +  (771 * 100))/314)
-#define W83782D_INIT_IN_MIN_6 \
-        ((W83782D_INIT_IN_MIN_6_TMP > 0) ? W83782D_INIT_IN_MIN_6_TMP : 0)
-#define W83782D_INIT_IN_MAX_6 \
-        ((( -500 * (100 - W83781D_INIT_IN_PERCENTAGE)) +  (771 * 100))/314)
-
-#define W83781D_INIT_FAN_MIN_1 3000
-#define W83781D_INIT_FAN_MIN_2 3000
-#define W83781D_INIT_FAN_MIN_3 3000
-
-#define W83781D_INIT_TEMP_OVER 600
-#define W83781D_INIT_TEMP_HYST 1270	/* must be 127 for ALARM to work */
-#define W83781D_INIT_TEMP2_OVER 600
-#define W83781D_INIT_TEMP2_HYST 500
-#define W83781D_INIT_TEMP3_OVER 600
-#define W83781D_INIT_TEMP3_HYST 500
-
 /* There are some complications in a module like this. First off, W83781D chips
    may be both present on the SMBus and the ISA bus, and we have to handle
    those cases separately at some places. Second, there might be several
@@ -359,6 +248,7 @@ static inline u8 DIV_TO_REG(long val, enum chips type)
    dynamically allocated, at the same time when a new w83781d client is
    allocated. */
 struct w83781d_data {
+	struct i2c_client client;
 	struct semaphore lock;
 	int sysctl_id;
 	enum chips type;
@@ -438,7 +328,6 @@ static void w83781d_sens(struct i2c_client *client, int operation,
 static void w83781d_rt(struct i2c_client *client, int operation,
 		       int ctl_name, int *nrels_mag, long *results);
 #endif
-static u16 swap_bytes(u16 val);
 
 static int w83781d_id = 0;
 
@@ -553,10 +442,6 @@ static ctl_table as99127f_dir_table_template[] = {
 	 &i2c_sysctl_real, NULL, &w83781d_alarms},
 	{W83781D_SYSCTL_BEEP, "beep", NULL, 0, 0644, NULL, &i2c_proc_real,
 	 &i2c_sysctl_real, NULL, &w83781d_beep},
-	{W83781D_SYSCTL_PWM1, "pwm1", NULL, 0, 0644, NULL, &i2c_proc_real,
-	 &i2c_sysctl_real, NULL, &w83781d_pwm},
-	{W83781D_SYSCTL_PWM2, "pwm2", NULL, 0, 0644, NULL, &i2c_proc_real,
-	 &i2c_sysctl_real, NULL, &w83781d_pwm},
 	{0}
 };
 
@@ -628,7 +513,7 @@ static ctl_table w83782d_isa_dir_table_template[] = {
 	 &i2c_sysctl_real, NULL, &w83781d_in},
 	{W83781D_SYSCTL_IN8, "in8", NULL, 0, 0644, NULL, &i2c_proc_real,
 	 &i2c_sysctl_real, NULL, &w83781d_in},
-{W83781D_SYSCTL_FAN1, "fan1", NULL, 0, 0644, NULL, &i2c_proc_real,
+	{W83781D_SYSCTL_FAN1, "fan1", NULL, 0, 0644, NULL, &i2c_proc_real,
 	 &i2c_sysctl_real, NULL, &w83781d_fan},
 	{W83781D_SYSCTL_FAN2, "fan2", NULL, 0, 0644, NULL, &i2c_proc_real,
 	 &i2c_sysctl_real, NULL, &w83781d_fan},
@@ -935,14 +820,12 @@ static int w83781d_detect(struct i2c_adapter *adapter, int address,
 	   client structure, even though we cannot fill it completely yet.
 	   But it allows us to access w83781d_{read,write}_value. */
 
-	if (!(new_client = kmalloc(sizeof(struct i2c_client) +
-				   sizeof(struct w83781d_data),
-				   GFP_KERNEL))) {
+	if (!(data = kmalloc(sizeof(struct w83781d_data), GFP_KERNEL))) {
 		err = -ENOMEM;
 		goto ERROR0;
 	}
 
-	data = (struct w83781d_data *) (new_client + 1);
+	new_client = &data->client;
 	new_client->addr = address;
 	init_MUTEX(&data->lock);
 	new_client->data = data;
@@ -966,15 +849,13 @@ static int w83781d_detect(struct i2c_adapter *adapter, int address,
 		val2 = w83781d_read_value(new_client, W83781D_REG_CHIPMAN);
 		/* Check for Winbond or Asus ID if in bank 0 */
 		if ((!(val1 & 0x07)) &&
-		    (((!(val1 & 0x80)) && (val2 != 0xa3) && (val2 != 0xc3) 
-			 && (val2 != 0x94))
-		     || ((val1 & 0x80) && (val2 != 0x5c) && (val2 != 0x12)
-			 && (val2 != 0x06)))) {
-			goto ERROR1;
+		    (((!(val1 & 0x80)) && (val2 != 0xa3) && (val2 != 0xc3))
+		     || ((val1 & 0x80) && (val2 != 0x5c) && (val2 != 0x12)))) {
 			err = -ENODEV;
+			goto ERROR1;
 		}
-		/* If Winbond SMBus, check address at 0x48. Asus doesn't support
-		   except maybe (hopefully) for the as99127f rev.2 */
+		/* If Winbond SMBus, check address at 0x48.
+		   Asus doesn't support, except for the as99127f rev.2 */
 		if ((!is_isa) && (((!(val1 & 0x80)) && (val2 == 0xa3)) ||
 				  ((val1 & 0x80) && (val2 == 0x5c)))) {
 			if (w83781d_read_value
@@ -998,7 +879,7 @@ static int w83781d_detect(struct i2c_adapter *adapter, int address,
 		val2 = w83781d_read_value(new_client, W83781D_REG_CHIPMAN);
 		if (val2 == 0x5c)
 			vendid = winbond;
-		else if ((val2 == 0x12) || (val2 == 0x06))
+		else if (val2 == 0x12)
 			vendid = asus;
 		else {
 			err = -ENODEV;
@@ -1012,9 +893,9 @@ static int w83781d_detect(struct i2c_adapter *adapter, int address,
 			kind = w83782d;
 		else if (val1 == 0x40 && vendid == winbond && !is_isa && address == 0x2d)
 			kind = w83783s;
-		else if (val1 == 0x20 && vendid == winbond)
+		else if (val1 == 0x21 && vendid == winbond)
 			kind = w83627hf;
-		else if (val1 == 0x70 && vendid == winbond && address >= 0x2c)
+		else if (val1 == 0x71 && vendid == winbond && address >= 0x2c)
 			kind = w83791d;
 		else if (val1 == 0x31 && !is_isa && address >= 0x28)
 			kind = as99127f;
@@ -1175,6 +1056,10 @@ static int w83781d_detect(struct i2c_adapter *adapter, int address,
 	}
 	data->sysctl_id = i;
 
+	/* Only PWM2 can be disabled */
+	for(i = 0; i < 4; i++)
+		data->pwmenable[i] = 1;
+
 	/* Initialize the chip */
 	w83781d_init_client(new_client);
 	return 0;
@@ -1203,7 +1088,7 @@ static int w83781d_detect(struct i2c_adapter *adapter, int address,
 	if (is_isa)
 		release_region(address, W83781D_EXTENT);
       ERROR1:
-	kfree(new_client);
+	kfree(data);
       ERROR0:
 	return err;
 }
@@ -1211,9 +1096,9 @@ static int w83781d_detect(struct i2c_adapter *adapter, int address,
 static int w83781d_detach_client(struct i2c_client *client)
 {
 	int err;
+	struct w83781d_data *data = client->data;
 
-	i2c_deregister_entry(((struct w83781d_data *) (client->data))->
-				 sysctl_id);
+	i2c_deregister_entry(data->sysctl_id);
 
 	if ((err = i2c_detach_client(client))) {
 		printk
@@ -1224,27 +1109,16 @@ static int w83781d_detach_client(struct i2c_client *client)
 	if(i2c_is_isa_client(client)) {
 		release_region(client->addr, W83781D_EXTENT);
 	} else {
-		i2c_detach_client(&
-				  (((struct
-				     w83781d_data *) (client->data))->
-				   lm75[0]));
-		if((((struct w83781d_data *) (client->data))->type) != w83783s)
-			i2c_detach_client(&
-				  (((struct
-				     w83781d_data *) (client->data))->
-				   lm75[1]));
-		kfree(((struct w83781d_data *) (client->data))->lm75);
+		i2c_detach_client(&(data->lm75[0]));
+		if (data->type != w83783s)
+			i2c_detach_client(&(data->lm75[1]));
+		kfree(data->lm75);
 	}
-	kfree(client);
+	kfree(data);
 
 	return 0;
 }
 
-static inline u16 swap_bytes(u16 val)
-{
-	return (val >> 8) | (val << 8);
-}
-
 /* The SMBus locks itself, usually, but nothing may access the Winbond between
    bank switches. ISA access must always be locked explicitly! 
    We ignore the W83781D BUSY flag at this moment - it could lead to deadlocks,
@@ -1299,23 +1173,17 @@ static int w83781d_read_value(struct i2c_client *client, u16 reg)
 			/* convert from ISA to LM75 I2C addresses */
 			switch (reg & 0xff) {
 			case 0x50: /* TEMP */
-				res =
-				    swap_bytes(i2c_smbus_read_word_data
-					       (cl, 0));
+				res = swab16(i2c_smbus_read_word_data(cl, 0));
 				break;
 			case 0x52: /* CONFIG */
 				res = i2c_smbus_read_byte_data(cl, 1);
 				break;
 			case 0x53: /* HYST */
-				res =
-				    swap_bytes(i2c_smbus_read_word_data
-					       (cl, 2));
+				res = swab16(i2c_smbus_read_word_data(cl, 2));
 				break;
 			case 0x55: /* OVER */
 			default:
-				res =
-				    swap_bytes(i2c_smbus_read_word_data
-					       (cl, 3));
+				res = swab16(i2c_smbus_read_word_data(cl, 3));
 				break;
 			}
 		}
@@ -1374,16 +1242,13 @@ static int w83781d_write_value(struct i2c_client *client, u16 reg, u16 value)
 			/* convert from ISA to LM75 I2C addresses */
 			switch (reg & 0xff) {
 			case 0x52: /* CONFIG */
-				i2c_smbus_write_byte_data(cl, 1,
-							  value & 0xff);
+				i2c_smbus_write_byte_data(cl, 1, value & 0xff);
 				break;
 			case 0x53: /* HYST */
-				i2c_smbus_write_word_data(cl, 2,
-							  swap_bytes(value));
+				i2c_smbus_write_word_data(cl, 2, swab16(value));
 				break;
 			case 0x55: /* OVER */
-				i2c_smbus_write_word_data(cl, 3,
-							  swap_bytes(value));
+				i2c_smbus_write_word_data(cl, 3, swab16(value));
 				break;
 			}
 		}
@@ -1399,7 +1264,7 @@ static int w83781d_write_value(struct i2c_client *client, u16 reg, u16 value)
 static void w83781d_init_client(struct i2c_client *client)
 {
 	struct w83781d_data *data = client->data;
-	int vid = 0, i, p;
+	int i, p;
 	int type = data->type;
 	u8 tmp;
 
@@ -1421,17 +1286,7 @@ static void w83781d_init_client(struct i2c_client *client)
 		w83781d_write_value(client, W83781D_REG_BEEP_INTS2, 0);
 	}
 
-	if (type != w83697hf) {
-		vid = w83781d_read_value(client, W83781D_REG_VID_FANDIV) & 0x0f;
-		vid |=
-		    (w83781d_read_value(client, W83781D_REG_CHIPID) & 0x01) << 4;
-                if (type == w83791d) {
-                        data->vrm = 92;
-                } else {
-		data->vrm = DEFAULT_VRM;
-                }
-		vid = vid_from_reg(vid, data->vrm);
-	}
+	data->vrm = (type == w83791d) ? 90 : 82;
 
 	if ((type != w83781d) && (type != as99127f)) {
 		tmp = w83781d_read_value(client, W83781D_REG_SCFG1);
@@ -1480,154 +1335,7 @@ static void w83781d_init_client(struct i2c_client *client)
 #endif				/* W83781D_RT */
 
 	if(init) {
-                if (type == w83791d) {
-                        w83781d_write_value(client, W83791D_REG_IN_MIN(0),
-                                            IN_TO_REG(W83781D_INIT_IN_MIN_0));
-                        w83781d_write_value(client, W83791D_REG_IN_MAX(0),
-                                            IN_TO_REG(W83781D_INIT_IN_MAX_0));
-                        w83781d_write_value(client, W83791D_REG_IN_MIN(1),
-                                            IN_TO_REG(W83781D_INIT_IN_MIN_1));
-                        w83781d_write_value(client, W83791D_REG_IN_MAX(1),
-                                            IN_TO_REG(W83781D_INIT_IN_MAX_1));
-                        w83781d_write_value(client, W83791D_REG_IN_MIN(2),
-                                            IN_TO_REG(W83781D_INIT_IN_MIN_2));
-                        w83781d_write_value(client, W83791D_REG_IN_MAX(2),
-                                            IN_TO_REG(W83781D_INIT_IN_MAX_2));
-                        w83781d_write_value(client, W83791D_REG_IN_MIN(3),
-                                            IN_TO_REG(W83781D_INIT_IN_MIN_3));
-                        w83781d_write_value(client, W83791D_REG_IN_MAX(3),
-                                            IN_TO_REG(W83781D_INIT_IN_MAX_3));
-                        w83781d_write_value(client, W83791D_REG_IN_MIN(4),
-                                            IN_TO_REG(W83781D_INIT_IN_MIN_4));
-                        w83781d_write_value(client, W83791D_REG_IN_MAX(4),
-                                            IN_TO_REG(W83781D_INIT_IN_MAX_4));
-                        w83781d_write_value(client, W83791D_REG_IN_MIN(5),
-                                            IN_TO_REG(W83781D_INIT_IN_MIN_5));
-                        w83781d_write_value(client, W83791D_REG_IN_MAX(5),
-                                            IN_TO_REG(W83781D_INIT_IN_MAX_5));
-                        w83781d_write_value(client, W83791D_REG_IN_MIN(6),
-                                            IN_TO_REG(W83781D_INIT_IN_MIN_6));
-                        w83781d_write_value(client, W83791D_REG_IN_MAX(6),
-                                            IN_TO_REG(W83781D_INIT_IN_MAX_6));
-                        w83781d_write_value(client, W83791D_REG_IN_MIN(7),
-                                            IN_TO_REG(W83781D_INIT_IN_MIN_7));
-                        w83781d_write_value(client, W83791D_REG_IN_MAX(7),
-                                            IN_TO_REG(W83781D_INIT_IN_MAX_7));
-                        w83781d_write_value(client, W83791D_REG_IN_MIN(8),
-                                            IN_TO_REG(W83781D_INIT_IN_MIN_8));
-                        w83781d_write_value(client, W83791D_REG_IN_MAX(8),
-                                            IN_TO_REG(W83781D_INIT_IN_MAX_8));
-                        w83781d_write_value(client, W83791D_REG_IN_MIN(9),
-                                            IN_TO_REG(W83781D_INIT_IN_MIN_9));
-                        w83781d_write_value(client, W83791D_REG_IN_MAX(9),
-                                            IN_TO_REG(W83781D_INIT_IN_MAX_9));
-                } else {
-		w83781d_write_value(client, W83781D_REG_IN_MIN(0),
-				    IN_TO_REG(W83781D_INIT_IN_MIN_0));
-		w83781d_write_value(client, W83781D_REG_IN_MAX(0),
-				    IN_TO_REG(W83781D_INIT_IN_MAX_0));
-		if (type != w83783s && type != w83697hf) {
-			w83781d_write_value(client, W83781D_REG_IN_MIN(1),
-					    IN_TO_REG(W83781D_INIT_IN_MIN_1));
-			w83781d_write_value(client, W83781D_REG_IN_MAX(1),
-					    IN_TO_REG(W83781D_INIT_IN_MAX_1));
-		}
-
-		w83781d_write_value(client, W83781D_REG_IN_MIN(2),
-				    IN_TO_REG(W83781D_INIT_IN_MIN_2));
-		w83781d_write_value(client, W83781D_REG_IN_MAX(2),
-				    IN_TO_REG(W83781D_INIT_IN_MAX_2));
-		w83781d_write_value(client, W83781D_REG_IN_MIN(3),
-				    IN_TO_REG(W83781D_INIT_IN_MIN_3));
-		w83781d_write_value(client, W83781D_REG_IN_MAX(3),
-				    IN_TO_REG(W83781D_INIT_IN_MAX_3));
-		w83781d_write_value(client, W83781D_REG_IN_MIN(4),
-				    IN_TO_REG(W83781D_INIT_IN_MIN_4));
-		w83781d_write_value(client, W83781D_REG_IN_MAX(4),
-				    IN_TO_REG(W83781D_INIT_IN_MAX_4));
-		if (type == w83781d || type == as99127f) {
-			w83781d_write_value(client, W83781D_REG_IN_MIN(5),
-					    IN_TO_REG(W83781D_INIT_IN_MIN_5));
-			w83781d_write_value(client, W83781D_REG_IN_MAX(5),
-					    IN_TO_REG(W83781D_INIT_IN_MAX_5));
-		} else {
-			w83781d_write_value(client, W83781D_REG_IN_MIN(5),
-					    IN_TO_REG(W83782D_INIT_IN_MIN_5));
-			w83781d_write_value(client, W83781D_REG_IN_MAX(5),
-					    IN_TO_REG(W83782D_INIT_IN_MAX_5));
-		}
-		if (type == w83781d || type == as99127f) {
-			w83781d_write_value(client, W83781D_REG_IN_MIN(6),
-					    IN_TO_REG(W83781D_INIT_IN_MIN_6));
-			w83781d_write_value(client, W83781D_REG_IN_MAX(6),
-					    IN_TO_REG(W83781D_INIT_IN_MAX_6));
-		} else {
-			w83781d_write_value(client, W83781D_REG_IN_MIN(6),
-					    IN_TO_REG(W83782D_INIT_IN_MIN_6));
-			w83781d_write_value(client, W83781D_REG_IN_MAX(6),
-					    IN_TO_REG(W83782D_INIT_IN_MAX_6));
-		}
-		if ((type == w83782d) || (type == w83627hf) ||
-		    (type == w83697hf)) {
-			w83781d_write_value(client, W83781D_REG_IN_MIN(7),
-					    IN_TO_REG(W83781D_INIT_IN_MIN_7));
-			w83781d_write_value(client, W83781D_REG_IN_MAX(7),
-					    IN_TO_REG(W83781D_INIT_IN_MAX_7));
-			w83781d_write_value(client, W83781D_REG_IN_MIN(8),
-					    IN_TO_REG(W83781D_INIT_IN_MIN_8));
-			w83781d_write_value(client, W83781D_REG_IN_MAX(8),
-					    IN_TO_REG(W83781D_INIT_IN_MAX_8));
-			w83781d_write_value(client, W83781D_REG_VBAT,
-			    (w83781d_read_value(client, W83781D_REG_VBAT) | 0x01));
-		}
-                }
-
-		w83781d_write_value(client, W83781D_REG_FAN_MIN(1),
-				    FAN_TO_REG(W83781D_INIT_FAN_MIN_1, 2));
-		w83781d_write_value(client, W83781D_REG_FAN_MIN(2),
-				    FAN_TO_REG(W83781D_INIT_FAN_MIN_2, 2));
-		if (type != w83697hf) {
-			w83781d_write_value(client, W83781D_REG_FAN_MIN(3),
-				    FAN_TO_REG(W83781D_INIT_FAN_MIN_3, 2));
-		}
-
-		w83781d_write_value(client, W83781D_REG_TEMP_OVER,
-				    TEMP_TO_REG(W83781D_INIT_TEMP_OVER));
-		w83781d_write_value(client, W83781D_REG_TEMP_HYST,
-				    TEMP_TO_REG(W83781D_INIT_TEMP_HYST));
-
-		if (type == as99127f) {
-			w83781d_write_value(client, W83781D_REG_TEMP2_OVER,
-					    AS99127_TEMP_ADD_TO_REG
-					    (W83781D_INIT_TEMP2_OVER));
-			w83781d_write_value(client, W83781D_REG_TEMP2_HYST,
-					    AS99127_TEMP_ADD_TO_REG
-					    (W83781D_INIT_TEMP2_HYST));
-		} else {
-			w83781d_write_value(client, W83781D_REG_TEMP2_OVER,
-					    TEMP_ADD_TO_REG
-					    (W83781D_INIT_TEMP2_OVER));
-			w83781d_write_value(client, W83781D_REG_TEMP2_HYST,
-					    TEMP_ADD_TO_REG
-					    (W83781D_INIT_TEMP2_HYST));
-		}
 		w83781d_write_value(client, W83781D_REG_TEMP2_CONFIG, 0x00);
-
-		if (type == as99127f) {
-			w83781d_write_value(client, W83781D_REG_TEMP3_OVER,
-					    AS99127_TEMP_ADD_TO_REG
-					    (W83781D_INIT_TEMP3_OVER));
-			w83781d_write_value(client, W83781D_REG_TEMP3_HYST,
-					    AS99127_TEMP_ADD_TO_REG
-					    (W83781D_INIT_TEMP3_HYST));
-		} else if (type != w83783s && type != w83697hf) {
-			w83781d_write_value(client, W83781D_REG_TEMP3_OVER,
-					    TEMP_ADD_TO_REG
-					    (W83781D_INIT_TEMP3_OVER));
-			w83781d_write_value(client, W83781D_REG_TEMP3_HYST,
-					    TEMP_ADD_TO_REG
-					    (W83781D_INIT_TEMP3_HYST));
-		}
 		if (type != w83783s && type != w83697hf) {
 			w83781d_write_value(client, W83781D_REG_TEMP3_CONFIG,
 					    0x00);
@@ -1635,9 +1343,10 @@ static void w83781d_init_client(struct i2c_client *client)
 		if (type != w83781d) {
 			/* enable comparator mode for temp2 and temp3 so
 		           alarm indication will work correctly */
-			w83781d_write_value(client, W83781D_REG_IRQ, 0x41);
-			for(i = 0; i < 3; i++)
-				data->pwmenable[i] = 1;
+			i = w83781d_read_value(client, W83781D_REG_IRQ);
+			if (!(i & 0x40))
+				w83781d_write_value(client, W83781D_REG_IRQ,
+						    i | 0x40);
 		}
 	}
 
@@ -1698,20 +1407,22 @@ static void w83781d_update_client(struct i2c_client *client)
                                               W83781D_REG_FAN_MIN(i));
                        if (data->type != w83791d && i == 3) break;
                }
-               if (data->type != w83781d) {
+               if (data->type != w83781d && data->type != as99127f) {
                        for (i = 1; i <= 4; i++) {
                                data->pwm[i - 1] =
                                    w83781d_read_value(client,
 				             W83781D_REG_PWM(data->type, i));
                                if (((data->type == w83783s)
                                     || (data->type == w83627hf)
-                                    || (data->type == as99127f)
                                     || (data->type == w83697hf)
                                     || ((data->type == w83782d)
                                        && i2c_is_isa_client(client)))
                                    && i == 2)
                                        break;
                        }
+			/* Only PWM2 can be disabled */
+			data->pwmenable[1] = (w83781d_read_value(client,
+					      W83781D_REG_PWMCLK12) & 0x08) >> 3;
                }
 
                data->temp = w83781d_read_value(client, W83781D_REG_TEMP);
@@ -1896,10 +1607,10 @@ void w83781d_temp_add(struct i2c_client *client, int operation,
 			    AS99127_TEMP_ADD_FROM_REG(data->temp_add[nr]);
 		} else {
 			results[0] =
-			    TEMP_ADD_FROM_REG(data->temp_add_over[nr]);
+			    LM75_TEMP_FROM_REG(data->temp_add_over[nr]);
 			results[1] =
-			    TEMP_ADD_FROM_REG(data->temp_add_hyst[nr]);
-			results[2] = TEMP_ADD_FROM_REG(data->temp_add[nr]);
+			    LM75_TEMP_FROM_REG(data->temp_add_hyst[nr]);
+			results[2] = LM75_TEMP_FROM_REG(data->temp_add[nr]);
 		}
 		*nrels_mag = 3;
 	} else if (operation == SENSORS_PROC_REAL_WRITE) {
@@ -1909,7 +1620,7 @@ void w83781d_temp_add(struct i2c_client *client, int operation,
 				    AS99127_TEMP_ADD_TO_REG(results[0]);
 			else
 				data->temp_add_over[nr] =
-				    TEMP_ADD_TO_REG(results[0]);
+				    LM75_TEMP_TO_REG(results[0]);
 			w83781d_write_value(client,
 					    nr ? W83781D_REG_TEMP3_OVER :
 					    W83781D_REG_TEMP2_OVER,
@@ -1921,7 +1632,7 @@ void w83781d_temp_add(struct i2c_client *client, int operation,
 				    AS99127_TEMP_ADD_TO_REG(results[1]);
 			else
 				data->temp_add_hyst[nr] =
-				    TEMP_ADD_TO_REG(results[1]);
+				    LM75_TEMP_TO_REG(results[1]);
 			w83781d_write_value(client,
 					    nr ? W83781D_REG_TEMP3_HYST :
 					    W83781D_REG_TEMP2_HYST,
@@ -1983,11 +1694,11 @@ void w83781d_beep(struct i2c_client *client, int operation, int ctl_name,
 	else if (operation == SENSORS_PROC_REAL_READ) {
 		w83781d_update_client(client);
 		results[0] = BEEP_ENABLE_FROM_REG(data->beep_enable);
-		results[1] = BEEPS_FROM_REG(data->beeps);
+		results[1] = BEEPS_FROM_REG(data->beeps, data->type);
 		*nrels_mag = 2;
 	} else if (operation == SENSORS_PROC_REAL_WRITE) {
 		if (*nrels_mag >= 2) {
-			data->beeps = BEEPS_TO_REG(results[1]);
+			data->beeps = BEEPS_TO_REG(results[1], data->type);
 			w83781d_write_value(client, W83781D_REG_BEEP_INTS1,
 					    data->beeps & 0xff);
 			if ((data->type != w83781d) &&
